<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\App;

use Amasty\Geoip\Model\Geolocation;
use Amasty\GeoipRedirect\Helper\Data;
use Exception;
use FiloBlu\ExtAmastyGeoipRedirect\Model\Logger\Logger;
use Magento\Customer\Model\Session;
use Magento\Framework\App;
use Magento\Framework\App\Area;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Request\Http as RequestHttp;
use Magento\Framework\App\Response\Http as ResponseHttp;
use Magento\Framework\App\State;
use Magento\Framework\AppInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Framework\ObjectManager\ConfigLoaderInterface;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\UrlInterface;
use Magento\Store\Api\StoreCookieManagerInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Geoip
 * @package FiloBlu\ExtAmastyGeoipRedirect\App
 */
class Geoip implements AppInterface
{
    /**
     * @var State
     */
    protected $state;

    /**
     * @var RequestHttp
     */
    protected $request;

    /**
     * @var ResponseHttp
     */
    protected $response;
    /**
     * @var \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data
     */
    protected $helper;
    /**
     * @var Logger
     */
    protected $logger;
    /**
     * @var \FiloBlu\MultiDomain\Helper\Data
     */
    protected $multiDomainHelper;
    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var Data
     */
    protected $geoipHelper;
    /**
     * @var UrlInterface
     */
    protected $urlBuilder;
    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;
    /**
     * @var Geolocation
     */
    protected $geolocation;
    /**
     * @var Session
     */
    protected $customerSession;
    /**
     * @var StoreCookieManagerInterface
     */
    protected $storeCookieManager;
    /**
     * @var ConfigLoaderInterface
     */
    protected $configLoader;
    /**
     * @var bool
     */
    protected $redirectAllowed;
    /**
     * Object manager
     *
     * @var ObjectManagerInterface
     */
    private $objectManager;

    /**
     * Geoip constructor.
     * @param State $state
     * @param RequestHttp $request
     * @param ResponseHttp $response
     * @param ObjectManagerInterface $objectManager
     * @param array $parameters
     */
    public function __construct(

        State $state,
        RequestHttp $request,
        ResponseHttp $response,
        ObjectManagerInterface $objectManager,
        array $parameters = []
    ) {
        $this->state = $state;
        $this->request = $request;
        $this->request->setParams($parameters);
        $this->response = $response;
        $this->objectManager = $objectManager;
    }

    /**
     * @return Console\Response|ResponseHttp|App\ResponseInterface|void
     * @throws LocalizedException
     */
    public function launch()
    {
        $this->state->setAreaCode(Area::AREA_FRONTEND);
        $configLoader = $this->objectManager->get(ConfigLoaderInterface::class);
        $this->objectManager->configure($configLoader->load(Area::AREA_FRONTEND));

        $this->helper = $this->objectManager->get(\FiloBlu\ExtAmastyGeoipRedirect\Helper\Data::class);
        $this->logger = $this->objectManager->get(Logger::class);
        $this->multiDomainHelper = $this->objectManager->get(\FiloBlu\MultiDomain\Helper\Data::class);
        $this->remoteAddress = $this->objectManager->get(RemoteAddress::class);
        $this->scopeConfig = $this->objectManager->get(ScopeConfigInterface::class);
        $this->geoipHelper = $this->objectManager->get(Data::class);
        $this->urlBuilder = $this->objectManager->get(UrlInterface::class);
        $this->storeManager = $this->objectManager->get(StoreManagerInterface::class);
        $this->geolocation = $this->objectManager->get(Geolocation::class);
        $this->customerSession = $this->objectManager->get(Session::class);
        $this->storeCookieManager = $this->objectManager->get(StoreCookieManagerInterface::class);

        $restrictionBypass = false;

        $request = $this->request;

        if ($request->isXmlHttpRequest()) {
            return;
        }

        $response = $this->response;

        $currentStoreId = $this->storeManager->getStore()->getId();
        $scopeStores = ScopeInterface::SCOPE_STORES;

        $redirectStatus = $this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_status', $scopeStores, $currentStoreId);

        $logEnabled = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log', $scopeStores, $currentStoreId);
        $logEnabledForCookie = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log_for_cookie', $scopeStores, $currentStoreId);
        $logEnabledForIps = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log_for_ip', $scopeStores, $currentStoreId);

        $ipRestriction = $this->scopeConfig->getValue('amgeoipredirect/restriction/ip_restriction');
        $currentIp = $this->remoteAddress->getRemoteAddress();
        $fakeIp = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_ip', $scopeStores, $currentStoreId);
        $fakeCountry = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_country', $scopeStores, $currentStoreId);

        if ($logEnabled && $logEnabledForIps && !$currentIp) {
            $this->logger->info('Can\'t get ip');
        }

        if ($logEnabled && $logEnabledForIps && strpos($logEnabledForIps, $currentIp) === false && $logEnabledForIps !== $currentIp) {
                $logEnabled = false;
        }

        if ($logEnabled && $logEnabledForCookie && !$this->helper->getCanLogCookie()) {
            $logEnabled = false;
        }

        $this->redirectAllowed = $this->helper->applyLogic($request, $logEnabled, $this->urlBuilder->getCurrentUrl(), $this->logger);

        if ($logEnabled) {
            $this->logger->info(' #### START REQUEST #### ' . $this->urlBuilder->getCurrentUrl() . ' controller->' . $request->getControllerModule());
        }

        if ($logEnabled) {
            $this->logger->info("Current Ip: $currentIp");

            if ($fakeIp) {
                $this->logger->info("Fake Ip: $fakeIp");
            }

            if ($fakeCountry) {
                $this->logger->info("Fake Country: $fakeCountry");
            }
        }

        if ($fakeIp) {
            $currentIp = $fakeIp;
        }

        if (!empty($ipRestriction)) {
            $ipRestriction = array_map('rtrim', explode(PHP_EOL, $ipRestriction));
            foreach ($ipRestriction as $ip) {
                if ($currentIp && $currentIp === $ip) {
                    if ($logEnabled) {
                        $this->logger->info("No Redirect For Ip restriction {$ip}");
                    }
                    $restrictionBypass = true;
                }
            }
        }
        $isApi = $request->getControllerModule() === 'Mage_Api';
        if ($isApi || !$this->geoipHelper->isModuleOutputEnabled('Amasty_Geoip')) {
            if ($logEnabled) {
                $this->logger->info('No Redirect For No Output enabled Module or API Call');
            }
            $restrictionBypass = true;
        }

        $userAgent = $request->getHeader('USER_AGENT');
        $userAgentsIgnore = $this->scopeConfig->getValue('amgeoipredirect/restriction/user_agents_ignore');
        if (!empty($userAgentsIgnore)) {
            $userAgentsIgnore = explode(',', $userAgentsIgnore);
            $userAgentsIgnore = array_map('trim', $userAgentsIgnore);
            foreach ($userAgentsIgnore as $agent) {
                if ($userAgent && $agent && stripos($userAgent, $agent) !== false) {
                    if ($logEnabled) {
                        $this->logger->info("No Redirect For Not Allowed User agent '{$userAgent}'");
                    }
                    $restrictionBypass = true;
                }
            }
        }

        if ($logEnabled) {
            if ($this->redirectAllowed) {
                $this->logger->info('redirect is allowed');
            } else {
                $this->logger->info('redirect is not allowed');
            }
        }

        if (!$restrictionBypass && $this->redirectAllowed && $this->scopeConfig->getValue('amgeoipredirect/general/enable', $scopeStores, $currentStoreId) && !$this->helper->notGeolocalizedRequest($request)) {
            $country_header_config = $this->scopeConfig->getValue('amgeoipredirect/restriction/geolocation_header');

            if (!$request->getHeader($country_header_config)) {
                $location = $this->geolocation->locate($currentIp);
                $country = $location->getCountry();
            } else {
                $country = $request->getHeader($country_header_config);
            }

            if ($fakeCountry) {
                $country = $fakeCountry;
            }

            if ($logEnabled) {
                $this->logger->info("Current Country: {$country}");
            }

            if ($country) {
                $this->helper->setLocalizedCountryCookie($country);
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/restriction/first_visit_redirect')) {
                $getAmYetRedirectStore = $this->helper->getStoreCookie();
                $getAmYetRedirectCurrency = $this->helper->getCurrencyCookie();
                $getAmYetRedirectUrl = $this->helper->getUrlCookie();
            } else {
                $getAmYetRedirectStore = 0;
                $getAmYetRedirectCurrency = 0;
                $getAmYetRedirectUrl = 0;
            }

            if (!$getAmYetRedirectUrl && $this->scopeConfig->getValue('amgeoipredirect/country_url/enable_url')) {

                $urlMapping = $this->helper->getCountryUrlMapping($currentStoreId);
                $currentUrl = $this->urlBuilder->getCurrentUrl();
                foreach ($urlMapping as $countries => $url) {
                    if ($url !== $currentUrl && strpos($countries, $country) !== false) {
                        $this->helper->setUrlCookie();
                        $url = preg_replace('/&?geolocation=[^&]&*/', '', $url);
                        $response->setRedirect($url, $redirectStatus);
                        $response->sendResponse();
                        exit;
                    }
                }
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/country_store/enable_store')) {
                $redirect_always_from_slash = $this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_always_from_slash');
                $currentPath = trim(parse_url($this->urlBuilder->getCurrentUrl(), PHP_URL_PATH));

                if ($getAmYetRedirectStore) {
                    $can_redirect = true;

                    if (!$this->helper->geolocationRequest($request) && $getAmYetRedirectStore == $this->storeManager->getStore()->getId()) {
                        $can_redirect = false;
                    }
                    if ($redirect_always_from_slash && ($currentPath == '/' || $currentPath == '/geoip.php')) {
                        $can_redirect = true;
                    }

                    if ($can_redirect) {
                        $currentStoreUrl = str_replace('&amp;', '&', $this->storeManager->getStore($getAmYetRedirectStore)->getCurrentUrl(false));
                        $redirectStoreUrl = trim($currentStoreUrl);

                        if ($logEnabled) {
                            $this->logger->info("Cookie is set, redirect to -> {$redirectStoreUrl}");
                        }

                        $this->storeCookieManager->setStoreCookie($this->storeManager->getStore($getAmYetRedirectStore));
                        $redirectStoreUrl = preg_replace('/&?geolocation=[^&]&*/', '', $redirectStoreUrl);
                        $response->setRedirect($redirectStoreUrl, $redirectStatus);
                        $response->sendResponse();
                        exit;
                    } else {
                        if ($logEnabled) {
                            $this->logger->info('Cookie is set, store is correct , no redirect required');
                        }
                    }
                } else {
                    if ($country) {
                        $this->helper->setCountryCookie($country);

                        if ($logEnabled) {
                            $this->logger->info("Set country cookie -> {$country}");
                        }
                    } else {
                        $countryId = $this->scopeConfig->getValue('general/country/default', $scopeStores);
                        $this->helper->setCountryCookie($countryId);
                        if ($logEnabled) {
                            $this->logger->info("Set default country cookie -> {$country}");
                        }
                    }

                    $allStores = $this->storeManager->getStores();

                    $currentGroup = $this->multiDomainHelper->getCurrentGroup();

                    foreach ($allStores as $store) {
                        if (!$store->isActive() || ($currentGroup && $currentGroup != $this->multiDomainHelper->getCurrentGroup($store->getWebsite()->getCode()))) {
                            continue;
                        }

                        $currentStoreUrl = str_replace('&amp;', '&', $store->getCurrentUrl(false));
                        $redirectStoreUrl = trim($currentStoreUrl);
                        $countries = $this->scopeConfig->getValue('amgeoipredirect/country_store/affected_countries', $scopeStores, $store->getId());
                        if (!$this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_between_websites')) {
                            $useMultistores = $store->getWebsiteId() == $this->storeManager->getStore()->getWebsiteId();
                        } else {
                            $useMultistores = true;
                        }

                        $can_redirect = true;

                        if ($store->getId() == $currentStoreId && !$this->helper->geolocationRequest($request)) {
                            $can_redirect = false;
                        }

                        if ($redirect_always_from_slash && $currentPath === ' /') {
                            $can_redirect = true;
                        }

                        if (
                            $country && $countries && strpos($countries, $country) !== false && $can_redirect && $useMultistores
                        ) {
                            if ($logEnabled) {
                                $this->logger->info("Geolocalized, redirect to '{$redirectStoreUrl}'");
                            }

                            $this->helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            $redirectStoreUrl = preg_replace('/&?geolocation=[^&]&*/', '', $redirectStoreUrl);
                            $response->setRedirect($redirectStoreUrl, $redirectStatus);
                            $response->sendResponse();
                            exit;
                        }

                        if (
                            $country && $countries && strpos($countries, $country) !== false && $store->getId() == $currentStoreId
                        ) {
                            if ($logEnabled) {
                                $this->logger->info('Already in the right website, set only cookie no redirect necessary');
                            }

                            $this->helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            break;
                        }
                    }
                }
            }

            if (!$getAmYetRedirectCurrency && $this->scopeConfig->getValue('amgeoipredirect/country_currency/enable_currency')) {
                $currencyMapping = $this->helper->getCountryCurrencyMapping($currentStoreId);
                foreach ($currencyMapping as $countries => $currency) {
                    if (strpos($countries, $country) !== false && $this->storeManager->getStore()->getCurrentCurrencyCode() != $currency) {
                        $this->helper->setCurrencyCookie();
                        $this->storeManager->getStore()->setCurrentCurrencyCode($currency);
                    }
                }
            }
        }

        if ($this->scopeConfig->getValue('amgeoipredirect/general/enable', $scopeStores, $currentStoreId) && $this->redirectAllowed && !$this->helper->notGeolocalizedRequest($request)) {
            $currentUrl = $this->urlBuilder->getCurrentUrl();
            $currentUrl = preg_replace('/geoip.php/', '', $currentUrl);
            $currentUrl .= (parse_url($currentUrl, PHP_URL_QUERY) ? '&' : '?') . 'notgeolocalized=1';
            $response->setRedirect($currentUrl, $redirectStatus);
            $response->sendResponse();
            exit;
        }

        if (!$this->scopeConfig->getValue('amgeoipredirect/general/enable', $scopeStores, $currentStoreId)) {
            $response->setRedirect($this->storeManager->getStore()->getBaseUrl(), $redirectStatus);
            $response->sendResponse();
            exit;
        }

        if ($logEnabled) {
            $this->logger->info(' #### END REQUEST #### ');
        }

        return $response;
    }

    /**
     * {@inheritdoc}
     */
    public function catchException(App\Bootstrap $bootstrap, Exception $exception)
    {
        return false;
    }
}
