<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Controller\Index;

use FiloBlu\Newsletter\Helper\Data;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\View\Page\Config;
use Magento\Framework\View\Result\PageFactory;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 *
 */
class Landing extends Action
{

    /** @var \Magento\Framework\View\Result\PageFactory */
    protected $resultPageFactory;
    protected $_helper;
    protected $layoutFactory;
    protected $storeManager;
    protected $scopeConfig;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \FiloBlu\Newsletter\Helper\Data $helper
     * @param \Magento\Framework\View\Page\Config $layoutFactory
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        Context $context,
        StoreManagerInterface $storeManager,
        PageFactory $resultPageFactory,
        Data $helper,
        Config $layoutFactory,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->storeManager = $storeManager;
        $this->resultPageFactory = $resultPageFactory;
        $this->_helper = $helper;
        $this->layoutFactory = $layoutFactory;
        $this->scopeConfig = $scopeConfig;

        parent::__construct($context);
    }

    public function execute()
    {
        $currentStoreId = $this->storeManager->getStore()->getId();

        if (!$this->scopeConfig->getValue(
                'amgeoipredirect/general/enable',
                ScopeInterface::SCOPE_STORES,
                $currentStoreId
            ) || !$this->scopeConfig->getValue(
                'amgeoipredirect/landing/enable_landing',
                ScopeInterface::SCOPE_STORES,
                $currentStoreId
            )) {
            return null;
        }

        $this->layoutFactory->setRobots(
            $this->scopeConfig->getValue(
                'amgeoipredirect/landing/landing_robots',
                ScopeInterface::SCOPE_STORES,
                $currentStoreId
            )
        );
        $resultPage = $this->resultPageFactory->create();
        $this->getResponse()->setNoCacheHeaders();
        $resultPage->getConfig()->getTitle()->set(
            $this->scopeConfig->getValue(
                'amgeoipredirect/landing/landing_title',
                ScopeInterface::SCOPE_STORES,
                $currentStoreId
            )
        );
        $resultPage->getConfig()->setDescription(
            $this->scopeConfig->getValue(
                'amgeoipredirect/landing/landing_description',
                ScopeInterface::SCOPE_STORES,
                $currentStoreId
            )
        );

        $hreflangs = $this->_helper->getLandingHrefLang($currentStoreId);

        if ($hreflangs && count($hreflangs)) {
            foreach ($hreflangs as $hreflang) {
                $resultPage->getConfig()->addRemotePageAsset(
                    $hreflang['href'], 'alternate',
                    ['attributes' => ['rel' => 'alternate', 'hreflang' => $hreflang['hreflang']]]
                );
            }
        }

        return $resultPage;
    }

}
