<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\CustomerData;

use FiloBlu\ExtAmastyGeoipRedirect\Helper\Data;
use Magento\Customer\CustomerData\SectionSourceInterface;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Stdlib\Cookie\CookieSizeLimitReachedException;
use Magento\Framework\Stdlib\Cookie\FailureToSendException;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class GeoipDataMisfits
 * @package FiloBlu\ExtAmastyGeoipRedirect\CustomerData
 */
class GeoipDataMisfits implements SectionSourceInterface
{
    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * GeoipDataMisfits constructor.
     * @param Data $helper
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        Data $helper,
        StoreManagerInterface $storeManager
    ) {
        $this->helper = $helper;
        $this->storeManager = $storeManager;
    }

    /**
     * @return array
     * @throws InputException
     * @throws NoSuchEntityException
     * @throws CookieSizeLimitReachedException
     * @throws FailureToSendException
     */
    public function getSectionData()
    {
        $geoData = $this->helper->getGeoData();
        $geoStore = $geoData->getStore();
        $store = $this->storeManager->getStore();

        return [
            'geo_country_code' => $geoData->getCountryCode(),
            'geo_country_name' => $geoData->getCountryName(),
            'geo_store_id' => $geoStore ? $geoStore->getStoreId() : null,
            'geo_store_url' => $geoStore ? $geoStore->getBaseUrl() : null,
            'current_website_id' => $store->getWebsiteId(),
            'current_country_name' => $this->helper->getCurrentCountryName('en_US'),
            'must_show' => $geoStore && $store->getWebsiteId() !== $geoStore->getWebsiteId(),
        ];
    }
}
